# HG changeset patch
# User xuefeng <xuefeng@multicorewareinc.com>
# Date 1476262899 -28800
# Node ID 0e9fb7dfba5a110ff91161e1901fa83cd88f16bc
# Parent  c64393b415adf0e8901b5e9952d456871155abd6
Fix bug that over-writing PPS QP causing inconsisent output and optimise this method.

diff -r c64393b415ad -r 0e9fb7dfba5a source/encoder/encoder.cpp
--- a/source/encoder/encoder.cpp	Thu Oct 06 13:55:11 2016 +0530
+++ b/source/encoder/encoder.cpp	Wed Oct 12 17:01:39 2016 +0800
@@ -76,6 +76,7 @@
     m_offsetEmergency = NULL;
     m_iFrameNum = 0;
     m_iPPSQpMinus26 = 0;
+    m_iLastSliceQp = 0;
     for (int i = 0; i < X265_MAX_FRAME_THREADS; i++)
         m_frameEncoder[i] = NULL;
 
@@ -874,7 +875,7 @@
             if( frameEnc->m_lowres.bKeyframe && m_param->bRepeatHeaders )
             {
                 ScopedLock qpLock( m_sliceQpLock );
-                if( m_iFrameNum >= m_param->frameNumThreads )
+                if( m_iFrameNum > 0 )
                 {
                     //Search the least cost
                     int64_t iLeastCost = m_iBitsCostSum[0];
@@ -888,8 +889,12 @@
                         }
                     }
 
-                    m_iPPSQpMinus26 = (iLeastId + 1) - 26;
-
+                    /* If last slice Qp is close to (26 + m_iPPSQpMinus26) or outputs is all I-frame video,
+                       we don't need to change m_iPPSQpMinus26. */
+                    if( (abs( m_iLastSliceQp - (26 + m_iPPSQpMinus26)) > 1) && (m_iFrameNum > 1) )
+                    {
+                        m_iPPSQpMinus26 = (iLeastId + 1) - 26;
+                    }
                     m_iFrameNum = 0;
                 }
 
diff -r c64393b415ad -r 0e9fb7dfba5a source/encoder/encoder.h
--- a/source/encoder/encoder.h	Thu Oct 06 13:55:11 2016 +0530
+++ b/source/encoder/encoder.h	Wed Oct 12 17:01:39 2016 +0800
@@ -139,9 +139,10 @@
 
     /* For optimising slice QP */
     Lock               m_sliceQpLock;
-    int                m_iFrameNum;
+    int                m_iFrameNum;   
+    int                m_iPPSQpMinus26;
+    int                m_iLastSliceQp;
     int64_t            m_iBitsCostSum[QP_MAX_MAX + 1];
-    int                m_iPPSQpMinus26;
 
     Encoder();
     ~Encoder() {}
diff -r c64393b415ad -r 0e9fb7dfba5a source/encoder/frameencoder.cpp
--- a/source/encoder/frameencoder.cpp	Thu Oct 06 13:55:11 2016 +0530
+++ b/source/encoder/frameencoder.cpp	Wed Oct 12 17:01:39 2016 +0800
@@ -473,6 +473,7 @@
         }
 
         m_top->m_iFrameNum++;
+        m_top->m_iLastSliceQp = slice->m_sliceQp;
     }
 
     m_initSliceContext.resetEntropy(*slice);
